﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.IO;
using OfficeOpenXml;
using System.Data;
using DocumentFormat.OpenXml;
using DocumentFormat.OpenXml.Packaging;
using DocumentFormat.OpenXml.Spreadsheet;

namespace WebAppExcelUpload
{
    public partial class UploadCommentFile : System.Web.UI.Page
    {
        protected void Page_Load(object sender, EventArgs e)
        {

        }

        protected void btnsubmit_Click(object sender, EventArgs e)
        {
            try
            {
                String path = String.Empty;
                if (xclFileUpload.HasFile)
                {
                    path = Server.MapPath("~\\Temp\\") + xclFileUpload.FileName;
                    if (File.Exists(path))
                    {
                        File.Delete(path);
                    }
                    xclFileUpload.SaveAs(path);
                    readExcelFile(path, true);
                }
            }
            catch (Exception ex)
            {
                AppCode.Common.errorLog(System.Reflection.MethodBase.GetCurrentMethod().Name, "WebAppExcelUpload", ex);
            }
        }

        private bool readExcelFile(String path, bool otherway)
        {
            try
            {
                String sheetName = "";
                using (SpreadsheetDocument doc = SpreadsheetDocument.Open(path, true))
                {
                    if (doc.WorkbookPart.WorksheetParts.Count() > 0)
                    {
                        SharedStringTable sst = doc.WorkbookPart.SharedStringTablePart.SharedStringTable;
                        DataTable odt = generateExcelDataTable();
                        foreach (Sheet s in doc.WorkbookPart.Workbook.Sheets)
                        {
                            if (s.Name.ToString().ToUpper().Contains("SUMMARY"))
                                continue;
                            else
                            {
                               sheetName = s.Name;
                               WorksheetPart wsp=  doc.WorkbookPart.GetPartById(s.Id) as WorksheetPart;
                                if (wsp != null)
                                {
                                    SheetData sd=wsp.Worksheet.Elements<SheetData>().FirstOrDefault();
                                    if (sd.HasChildren)
                                    {
                                        foreach (Row r in sd.Elements<Row>().Skip(1))
                                        {
                                            DataRow odr = odt.NewRow();
                                            odr["BatchId"] = sheetName;
                                            var cells = r.Elements<Cell>();
                                            for (int i= 0; i< 6; i++)
                                            {
                                                Cell c = cells.ElementAt<Cell>(i);
                                                if (c.DataType != null && c.DataType == CellValues.SharedString)
                                                {
                                                    fillDataRow(i + 1, odr, sst.ChildElements[Convert.ToInt16(c.CellValue.InnerText)].InnerText.ToString());
                                                }
                                                else if (c.CellValue != null)
                                                {
                                                    fillDataRow(i + 1, odr, c.CellValue.Text);
                                                }
                                                else {
                                                    AppCode.Common.saveLog(sheetName, new Exception(String.Format("Invalid data at RowNumber {0} and column number {1}", r.RowIndex, i+1)));
                                                }
                                            }
                                            odt.Rows.Add(odr);
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
            catch (Exception)
            {
                throw;
            }
            return otherway;
        }

        private void fillDataRow(int colNumber, DataRow odr, String value)
        {
            switch (colNumber)
            {
                case 1:
                    odr["RespId"] = value;
                    break;
                case 2:
                    odr["VerbatimQueId"] = value;
                    break;
                case 3:
                    odr["SourceText"] = value;
                    break;
                case 4:
                    odr["Category"] = value;
                    break;
                case 5:
                    odr["SentimentValue"] = value;
                    break;
                case 6:
                    odr["SentimentType"] = value;
                    break;
            }
        }

        private bool readExcelFile(String path)
        {
            bool flag = false;
            if (xclFileUpload.HasFile)
            {   
                if (File.Exists(path))
                {
                    File.Delete(path);
                }
                xclFileUpload.SaveAs(path);
                FileInfo xclFile = new FileInfo(path);
                ExcelPackage pck = new ExcelPackage(xclFile);
                ExcelWorksheets ws = pck.Workbook.Worksheets;
                int RowNumber = 2;
                int ColNumber = 1;
                DataTable odt = generateExcelDataTable();
                foreach (ExcelWorksheet eWS in ws)
                {
                    AppCode.Common.saveLog(eWS.Name, new Exception("Processing sheet start."));
                    if (eWS.Name.Contains("summary"))
                    { continue; }
                    else
                    {
                        while (Convert.ToString(eWS.Cells[RowNumber, 1].Text).Trim().Length > 0)  ////if response id is empty i.e. the end of excel.
                        {
                            DataRow dr = odt.NewRow();
                            dr["BatchId"] = eWS.Name;
                            for (ColNumber = 1; ColNumber <= 6; ColNumber++)
                            {
                                switch (ColNumber)
                                {
                                    case 1:
                                        dr["RespId"] = Convert.ToString(eWS.Cells[RowNumber, ColNumber].Text);
                                        break;
                                    case 2:
                                        dr["VerbatimQueId"] = Convert.ToString(eWS.Cells[RowNumber, ColNumber].Text);
                                        break;
                                    case 3:
                                        dr["SourceText"] = Convert.ToString(eWS.Cells[RowNumber, ColNumber].Text);
                                        break;
                                    case 4:
                                        dr["Category"] = Convert.ToString(eWS.Cells[RowNumber, ColNumber].Text);
                                        break;
                                    case 5:
                                        dr["SentimentValue"] = Convert.ToString(eWS.Cells[RowNumber, ColNumber].Text);
                                        break;
                                    case 6:
                                        dr["SentimentType"] = Convert.ToString(eWS.Cells[RowNumber, ColNumber].Text);
                                        break;
                                }
                            }
                            odt.Rows.Add(dr);
                            RowNumber++;
                        }
                    }
                    AppCode.Common.saveLog(eWS.Name, new Exception("Processing sheet End."));
                }
            }
            else
            {
                throw new Exception("file not found.");
            }
            return flag;
        }

        private DataTable generateExcelDataTable()
        {
            DataTable odt = new DataTable();
            try
            {
                DataColumn odcBatchID = new DataColumn() { ColumnName = "BatchId" };
                odt.Columns.Add(odcBatchID);
                DataColumn odcRespID = new DataColumn() { ColumnName="RespId"};
                odt.Columns.Add(odcRespID);
                DataColumn odcQueID = new DataColumn() { ColumnName = "VerbatimQueId" };
                odt.Columns.Add(odcQueID);
                DataColumn odcSrcText = new DataColumn() { ColumnName = "SourceText" };
                odt.Columns.Add(odcSrcText);
                DataColumn odcCategory = new DataColumn() { ColumnName = "Category" };
                odt.Columns.Add(odcCategory);
                DataColumn odcSubCategory = new DataColumn() { ColumnName = "SubCategory" };
                odt.Columns.Add(odcSubCategory);
                DataColumn odcSentimentValue = new DataColumn() { ColumnName = "SentimentValue" };
                odt.Columns.Add(odcSentimentValue);
                DataColumn odcSentimentType= new DataColumn() { ColumnName = "SentimentType" };
                odt.Columns.Add(odcSentimentType);
            }
            catch (Exception ex)
            {
                AppCode.Common.errorLog(System.Reflection.MethodBase.GetCurrentMethod().Name, "WebAppExcelUpload", ex);
                throw;
            }
            return odt;
        }
    }
}